@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -55,7 +55,7 @@ public:
                               "Content-Type", "text/plain");
             
             http_res->content_buf = 
-                odr_strdup(odr, "Welcome to YP2");
+                odr_strdup(odr, "Welcome to Metaproxy");
             http_res->content_len = strlen(http_res->content_buf);
             
             package.response() = gdu;
@@ -155,8 +155,9 @@ int main(int argc, char **argv)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
