@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -44,6 +44,8 @@ namespace metaproxy_1 {
     namespace filter {
         class Session_info {
             int dummy;
+        public:
+            Session_info() { dummy = 0; };
         };
         class BackendTest::Rep {
             friend class BackendTest;
@@ -232,7 +234,7 @@ void yf::BackendTest::process(Package &package) const
             Z_InitRequest *req = apdu_req->u.initRequest;
             Z_InitResponse *resp = apdu_res->u.initResponse;
 
-            resp->implementationName = "backend_test";
+            resp->implementationName = odr_strdup(odr, "backend_test");
             if (ODR_MASK_GET(req->options, Z_Options_namedResultSets))
                 m_p->m_support_named_result_sets = true;
             
@@ -397,8 +399,9 @@ extern "C" {
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
