@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -158,7 +158,13 @@ void yf::HttpFile::Rep::fetch_file(mp::Session &session,
     Z_HTTP_Response *hres = gdu->u.HTTP_Response;
     hres->content_len = sz;
     hres->content_buf = (char*) odr_malloc(o, hres->content_len);
-    fread(hres->content_buf, 1, hres->content_len, f);
+    if (fread(hres->content_buf, 1, hres->content_len, f) != 1)
+    {
+        fclose(f);
+        Z_GDU *gdu = o.create_HTTP_Response(session, req, 500);
+        package.response() = gdu;
+        return;
+    }
 
     fclose(f);
     
@@ -290,8 +296,9 @@ extern "C" {
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
