@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -22,6 +22,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <yaz/options.h>
 #include <yaz/daemon.h>
 
+#include <yaz/sc.h>
 #include <iostream>
 #include <stdexcept>
 #include <libxml/xinclude.h>
@@ -31,6 +32,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include "router_flexml.hpp"
 #include "factory_static.hpp"
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef WIN32
+#include <direct.h>
+#include <io.h>
+#include <process.h>
+#endif
+
 namespace mp = metaproxy_1;
 
 static void handler(void *data)
@@ -41,7 +51,9 @@ static void handler(void *data)
     pack.router(*routerp).move();
 }
     
-int main(int argc, char **argv)
+static int sc_main(
+    yaz_sc_t s,
+    int argc, char **argv)
 {
     try 
     {
@@ -52,7 +64,7 @@ int main(int argc, char **argv)
         const char *pidfile = 0;
         const char *uid = 0;
 
-        while ((ret = options("c{config}:Dh{help}l:p:u:V{version}X", 
+        while ((ret = options("c{config}:Dh{help}l:p:u:V{version}w:X", 
                               argv, argc, &arg)) != -2)
         {
             switch (ret)
@@ -72,7 +84,13 @@ int main(int argc, char **argv)
                     " -l f          log file f\n"
                     " -p f          pid file f\n"
                     " -u id         change uid to id\n"
+                    " -w dir        changes working directory to dir\n"
                     " -X            debug mode (no fork/daemon mode)\n"
+#ifdef WIN32
+                    " -install      install windows service\n"
+                    " -remove       remove windows service\n"
+#endif
+
                           << std::endl;
                 break;
             case 'l':
@@ -86,20 +104,32 @@ int main(int argc, char **argv)
                 break;
             case 'V':
                 std::cout << VERSION "\n";
-                std::exit(0);
+                return 0;
                 break;
+            case 'w':
+                if (
+#ifdef WIN32
+                    _chdir(arg)
+#else
+                    chdir(arg)
+#endif
+                   ) 
+                {
+                    std::cerr << "chdir " << arg << " failed" << std::endl;
+                    return 1;
+                }
             case 'X':
                 mode = YAZ_DAEMON_DEBUG;
                 break;
             case -1:
                 std::cerr << "bad option: " << arg << std::endl;
-                std::exit(1);
+                return 1;
             }
         }
         if (!fname)
         {
             std::cerr << "No configuration given; use -h for help\n";
-            std::exit(1);
+            return 1;
         }
 
         yaz_log(YLOG_LOG, "Metaproxy " VERSION " started");
@@ -110,39 +140,62 @@ int main(int argc, char **argv)
         
         if (!doc)
         {
-            std::cerr << "XML parsing failed\n";
-            std::exit(1);
+            yaz_log (YLOG_FATAL,"XML parsing failed");
+            return 1;
         }
         // and perform Xinclude then
-        if (xmlXIncludeProcess(doc) > 0) {
-            std::cerr << "processing XInclude directive\n";
+        int r = xmlXIncludeProcess(doc);
+        if (r == -1)
+        {
+            yaz_log(YLOG_FATAL, "XInclude processing failed");
+            return 1;
         }
         mp::FactoryStatic factory;
         mp::RouterFleXML router(doc, factory, false);
 
+        yaz_sc_running(s);
+
         yaz_daemon("metaproxy", mode, handler, &router, pidfile, uid);
     }
     catch (std::logic_error &e) {
-        std::cerr << "std::logic error: " << e.what() << "\n";
-        std::exit(1);
+        yaz_log (YLOG_FATAL,"std::logic error: %s" , e.what() );
+        return 1;
     }
     catch (std::runtime_error &e) {
-        std::cerr << "std::runtime error: " << e.what() << "\n";
-        std::exit(1);
+        yaz_log(YLOG_FATAL, "std::runtime error: %s" , e.what() );
+        return 1;
     }
     catch ( ... ) {
-        std::cerr << "Unknown Exception" << std::endl;
-        std::exit(1);
+        yaz_log(YLOG_FATAL, "Unknown Exception");
+        return 1;
     }
-    std::exit(0);
+    return 0;
 }
 
+static void sc_stop(yaz_sc_t s)
+{
+    return;
+}
+
+int main(int argc, char **argv)
+{
+    int ret;
+    yaz_sc_t s = yaz_sc_create("metaproxy", "metaproxy");
+
+    ret = yaz_sc_program(s, argc, argv, sc_main, sc_stop);
+
+    yaz_sc_destroy(&s);
+    exit(ret);
+}
+
+
 
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
