@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -43,14 +43,11 @@ public:
         }
        
         Z_GDU *gdu = package.request().get();
-        if (gdu)
+        if (gdu && gdu->which == Z_GDU_Z3950)
         {
             // std::cout << "Got PDU. Sending init response\n";
             mp::odr odr;
-            Z_APDU *apdu = zget_APDU(odr, Z_APDU_initResponse);
-            
-            apdu->u.initResponse->implementationName = "YP2/YAZ";
-            
+            Z_APDU *apdu = odr.create_initResponse(gdu->u.z3950, 0, 0);
             package.response() = apdu;
         }
         return package.move();
@@ -111,8 +108,9 @@ BOOST_AUTO_TEST_CASE( test_filter_log_2 )
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
