@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -38,7 +38,7 @@ class Worker
 {
     public:
         Worker(int nr = 0) 
-            :  m_nr(nr){};
+            :  m_nr(nr), m_id(0) {};
         
         void operator() (void) {
             for (int i=0; i < 100; ++i)
@@ -97,8 +97,9 @@ BOOST_AUTO_TEST_CASE( testsession2 )
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
