@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -42,7 +42,7 @@ int main(int argc, char **argv)
 	exit(1);
     }
     void *sym = dlsym(mod, argv[2]);
-    printf("sym=%p\n", sym);
+    cout << "sym=" << sym << endl;
     dlclose(mod);
     exit(0);
 #else
@@ -54,8 +54,9 @@ int main(int argc, char **argv)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
