@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2008 Index Data
+   Copyright (C) 2005-2009 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -478,6 +478,13 @@ Z_APDU *mp::odr::create_initResponse(const Z_APDU *in_apdu,
             zget_init_diagnostics(m_odr, error, addinfo);
         *apdu->u.initResponse->result = 0;
     }
+    apdu->u.initResponse->implementationName =
+        odr_prepend(m_odr, "Metaproxy",
+                    apdu->u.initResponse->implementationName);
+    apdu->u.initResponse->implementationVersion = 
+        odr_prepend(m_odr,
+                    VERSION, apdu->u.initResponse->implementationVersion);
+                   
     return apdu;
 }
 
@@ -633,8 +640,9 @@ Z_ReferenceId **mp_util::get_referenceId(const Z_APDU *apdu)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
- * c-file-style: "stroustrup"
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
+
